/*
    LogExport: Export Symbian call log to a file
	Copyright (C) 2007 Yohanes Nugroho

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "LogWriter.h"
#include <coemain.h>
#include <logview.h>		// CLogViewEvent
#include <logwrap.h>
#include <stdio.h>
#include <EscapeUtils.h>
#include <LogExport.rsg>
#include <eikprogi.h>
#include <tz.h>

CLogWriter::CLogWriter():CActive(CActive::EPriorityStandard)
{

}

void CLogWriter::ConstructL()
{
	iFs.Connect();
	iLogClient = CLogClient::NewL(iFs);
	iLogFilter = CLogFilter::NewL();
	iLogView = CLogViewEvent::NewL(*iLogClient);
	CActiveScheduler::Add(this);
}

CLogWriter::~CLogWriter()
{
	Cancel();
	delete iLogView;
	delete iLogClient;		
	delete iLogFilter;
	
	iFs.Close();
}

TBool CLogWriter::IsDone()
{
	return iDone;
}	

_LIT8(QUOTE, "\"");
_LIT8(CR, "\n");
_LIT8(COMMA, ",");

void CLogWriter::Export(TDesC &aFileName)
{
	iFs.MkDirAll(aFileName);
    User::LeaveIfError(iFile.Replace(iFs, aFileName, EFileWrite));
	TBool eventsInView = iLogView->SetFilterL(*iLogFilter, iStatus);
	iEngineState = ECreatingView;
	
	if (eventsInView)
	{
		// if there are events in the filtered view
		// issue the asynchronous request
		iProgressDialog = new (ELeave) CAknProgressDialog(reinterpret_cast<CEikDialog**>(&iProgressDialog));
		iProgressDialog->PrepareLC(R_PROGRESSNOTE);
		iProgressDialog->SetCallback(this);
		CEikProgressInfo* progressBar = iProgressDialog->GetProgressInfoL();
		progressBar->SetFinalValue(iLogView->CountL());
		iProgressDialog->RunLD();
		_LIT(F1, "Date");
		_LIT(F2, "Time");
		_LIT(F3, "Type");
		_LIT(F4, "RemoteParty");
		_LIT(F5, "Direction");
		_LIT(F6, "DurationType");
		_LIT(F7, "Duration");
		_LIT(F8, "Status");
		_LIT(F9, "Subject");
		_LIT(F10, "Number");
		_LIT(F11, "SentData");
		_LIT(F12, "RecvData");
		_LIT(F13, "MsgType");
		_LIT(F14, "MsgTotalPart");
		_LIT(F15, "MsgTotalSent");
		_LIT(F16, "MsgTotalDelivered");
		_LIT(F17, "MsgTotalFailed");
		_LIT(F18, "MsgTotalReceived");		
		
		WriteField(F1);
		WriteField(F2);
		WriteField(F3);
		WriteField(F4);
		WriteField(F5);
		WriteField(F6);
		WriteField(F7);
		WriteField(F8);
		WriteField(F9);
		WriteField(F10);
		WriteField(F11);
		WriteField(F12);
		WriteField(F13);
		WriteField(F14);
		WriteField(F15);
		WriteField(F16);
		WriteField(F17);
		WriteField(F18);
		iFile.Write(CR);
		SetActive();
		iDone = EFalse;
	} else {
		
		iDone = ETrue;
	}
	
}


void CLogWriter::DoCancel()
{
	iDone = ETrue;
	iFile.Close();
}


void CLogWriter::WriteField(const TDesC &aText, int addcomma)
{	
	HBufC8 *l = EscapeUtils::ConvertFromUnicodeToUtf8L(aText);
	TPtr8 des = l->Des();
	for (int i = 0; i<des.Length(); i++){
		if (des[i]=='"' || des[i]=='\r' || des[i]=='\n') {
			des[i] = ' ';
		}
	}
	
	
	iFile.Write(QUOTE);
	iFile.Write(*l);
	iFile.Write(QUOTE);
	if (addcomma) {
		iFile.Write(COMMA);
	} else {
		iFile.Write(CR);
	}	
	delete l;
}

void CLogWriter::WriteField(TInt number, int addcomma)
{
	TBuf<100> x;
	x.Zero();
	x.AppendNum(number);
	WriteField(x, addcomma);
}


void CLogWriter::DialogDismissedL(TInt /*aButtonId*/)
{
	iDone = ETrue;
	if (iProgressDialog) iProgressDialog->ProcessFinishedL();
}


void CLogWriter::RunL()
{
	if (iStatus != KErrNone) {
		if (iProgressDialog) iProgressDialog->ProcessFinishedL();
		return;
	}
	if (iDone) {
		if (iProgressDialog) iProgressDialog->ProcessFinishedL();
		return;
	}
	switch (iEngineState)
		{
		case ECreatingView:
			{
			// The filtered view has been successfully created
			// so issue a request to go to the first event
			iLogView->FirstL(iStatus);
			iEngineState = EAddingEntry;
			SetActive();
			break;
			}
		case EAddingEntry:
			{
			// Add the current event to the array of events
			const CLogEvent& event = iLogView->Event();
			TTime t = event.Time();
#ifdef USE_LOCAL_TIME			
			RTz tz;
			tz.Connect();
			tz.ConvertToLocalTime(t);
			tz.Close();
#endif			
			TBuf<100> x;
			x.Zero();
			TDateTime td = t.DateTime();
			x.Format(_L("%02d/%02d/%04d"), td.Day()+1, td.Month()+1, td.Year());
			WriteField(x);
			x.Zero();
			t.FormatL(x, _L("%H:%T"));
			WriteField(x);
				
			//WriteField(event.EventType().iUid);			
				
			_LIT8(VIDEOCALL, "VT");			
			if ((event.EventType().iUid==  KLogCallEventType) && (event.Data()==VIDEOCALL)) {
				WriteField(_L("Video Call"));
			} else {
				WriteField(event.Description());
			}
			
			WriteField(event.RemoteParty());			
			WriteField(event.Direction());	
			WriteField(event.DurationType());
			WriteField(event.Duration());
			WriteField(event.Status());			
			WriteField(event.Subject());			
			WriteField(event.Number());
			
			if (event.EventType().iUid==KLogPacketDataEventType || event.EventType().iUid==KLogDataEventType 
				|| event.EventType().iUid==0x1000595F){
				iFile.Write(event.Data()); //already in comma form
			}
			if (event.EventType().iUid==KLogShortMessageEventType) {
					WriteField(_L("0"));
					WriteField(_L("0"));
					const TInt * data = (const TInt *)event.Data().Ptr();
					WriteField(*data); data++; //TInt iType; // me
					WriteField(*data); data++; //TInt iTotal;   //
					WriteField(*data); data++;  //TInt iSent;   //
					WriteField(*data); data++;  //TInt iDelivered;
					WriteField(*data); data++;  //TInt iFailed;  //
					WriteField(*data); //TInt iReceived;
			}
			
			//iFile.Write(event.Data());
			iFile.Write(CR);
			// Are there any more entries
			if (iLogView->NextL(iStatus))
				{
				// if there is another entry, issue the request
				// to move to the next entry.
				SetActive();
				iProgressDialog->GetProgressInfoL()->IncrementAndDraw(1);
				} else {
					iDone = ETrue;
					iFile.Close();
					if (iProgressDialog) iProgressDialog->ProcessFinishedL();
				}
			}
		default:
			break;
	}	
}

TInt CLogWriter::RunError(TInt aError)
{
	iDone = ETrue;
	return KErrNone;
}
